#ifndef MPIPE_API_NET_H
#define MPIPE_API_NET_H

#include "mpipe_types.h"
#include "mpipe_common.h"

EXTERN_C_BEGIN

// описатель адреса для сокетных типов транспорта
typedef struct mpipe_net_socket_address_t
{
    enum mpipe_net_ip_version_t             version;                // версия ip адреса
    union
    {
        uint32_t                            address_v4;             // ip адрес версии IP4
        uint8_t                             address_v6[16];         // ip адрес версии Ip6
    };
} mpipe_net_socket_address_t;

// описатель конечной точки для сокета
typedef struct mpipe_net_socket_endpoint_t
{
    mpipe_net_socket_address_t              address;                // адрес сокета
    mpipe_net_socket_port_t                 port;                   // порт сокета
} mpipe_net_socket_endpoint_t;

// описатель конечной точки для серийного порта
typedef struct mpipe_net_serial_endpoint_t
{
    const char*                             port_name;              // имя порта
} mpipe_net_serial_endpoint_t;

// общий описатель конечной точки транспорта
typedef struct mpipe_net_endpoint_t
{
    enum mpipe_net_transport_id_t           transport_id;           // идентификатор транспорта
    union
    {
        mpipe_net_socket_endpoint_t         socket;
        mpipe_net_serial_endpoint_t         serial;
    }                                       ep;
} mpipe_net_endpoint_t;

// описатель tcp-соединения
typedef struct mpipe_net_tcp_connection_info_t
{
    enum mpipe_net_role_t                   role;                   // роль соединения
    mpipe_net_socket_endpoint_t             local_endpoint;         // локальный эндпоинт
    mpipe_net_socket_endpoint_t             remote_endpoint;        // удаленный эндпоинт
} mpipe_net_tcp_connection_info_t;

// описатель tcp-листенера
typedef struct mpipe_net_tcp_listener_info_t
{
    mpipe_net_socket_endpoint_t             local_endpoint;         // локальный эндпоинт
} mpipe_net_tcp_listener_info_t;

// общий описатель tcp-транспорта
typedef struct mpipe_net_tcp_info_t
{
    enum mpipe_net_link_type_t              link_type;              // тип транспорта - слушатель (listener)
    mpipe_net_stream_protocol_t             stream_protocol;        // потокол верхнего уровня для TCP
    union
    {
         mpipe_net_tcp_connection_info_t    connection;
         mpipe_net_tcp_listener_info_t      listener;
    };
} mpipe_net_tcp_info_t;

// описатель udp-транспорта
typedef struct mpipe_net_udp_info_t
{
    mpipe_net_socket_endpoint_t             local_endpoint;         // локальный эндпоинт
    mpipe_net_socket_endpoint_t             remote_endpoint;        // удаленный эндпоинт
} mpipe_net_udp_info_t;

// общий описатель экземпляра транспорта
typedef struct mpipe_net_transport_t
{
    mpipe_handle_t                          handle;                 // дескриптор транспорта
    enum mpipe_net_transport_id_t           transport_id;           // идентификатор транспорта
    enum mpipe_object_state_t               state;                  // состояние транспорта
    void*                                   user_data;              // пользовательский контекст
    union
    {
        mpipe_net_tcp_info_t                tcp;
        mpipe_net_udp_info_t                udp;
    }                                       transport_info;         // информация о транспорте
} mpipe_net_transport_t;

// описатель сетевого пакета - единица обмена информацией для транспорта
typedef struct mpipe_net_packet_t
{   
    size_t                                  id;                     // идентификатор пакета
    mpipe_timestamp_t                       timestamp;              // метка времени пакета (в наносекундных тиках)
    mpipe_data_t                            packet_data;            // данные пакета
    mpipe_net_endpoint_t                    endpoint;               // эндпоинт отправителья/получателя опицонально
} mpipe_net_packet_t;


typedef void (*mpipe_cb_transport_state_t)(const struct mpipe_net_transport_t* transport
                                           , enum mpipe_object_state_t state);

typedef mpipe_bool_t (*mpipe_cb_transport_accept_t)(const struct mpipe_net_transport_t* transport
                                                    , const struct mpipe_net_transport_t* acception_transport
                                                    , void** acception_transport_user_data
                                                    , struct mpipe_net_transport_cb_t* acception_transport_cb);

typedef void (*mpipe_cb_transport_packet_t)(const struct mpipe_net_transport_t* transport
                                            , const struct mpipe_net_packet_t* packet);

// блок описания функций обратного вызова длят танспорта
typedef struct mpipe_net_transport_cb_t
{
    mpipe_cb_transport_state_t              on_state;               // состояние соединения
    mpipe_cb_transport_accept_t             on_accept;              // входящее соединение для "слушающих" транспортов
    mpipe_cb_transport_packet_t             on_packet;              // входящий пакет

} mpipe_net_transport_cb_t;

// общие опции для всех типов сокетов
typedef struct mpipe_net_socket_options_t
{
    mpipe_bool_t                            reuse_address;
} mpipe_net_socket_options_t;

typedef struct mpipe_net_tcp_connection_params_t
{
    mpipe_net_socket_endpoint_t             local_endpoint;
    mpipe_net_socket_endpoint_t             remote_endpoint;
} mpipe_net_tcp_connection_params_t;

typedef struct mpipe_net_tcp_listener_params_t
{
    mpipe_net_socket_endpoint_t             local_endpoint;
} mpipe_net_tcp_listener_params_t;

// параметры tcp
typedef struct mpipe_net_tcp_params_t
{
    mpipe_net_link_type_t                   link_type;              // тип транспорта - слушатель (listener)
    mpipe_net_stream_protocol_t             stream_protocol;        // потокол верхнего уровня
    mpipe_net_socket_options_t              socket_options;
    union
    {
        mpipe_net_tcp_connection_params_t   connection;
        mpipe_net_tcp_listener_params_t     listener;
    }                                       params;
} mpipe_net_tcp_params_t;

// параметры udp
typedef struct mpipe_net_udp_params_t
{
    mpipe_net_socket_options_t              socket_options;
    mpipe_net_socket_endpoint_t             local_endpoint;
    mpipe_net_socket_endpoint_t             remote_endpoint;
} mpipe_net_udp_params_t;

// прототип возможно будущего функционала для работы с последовательными портами (в т.ч. виртуальными)
typedef struct mpipe_net_serial_params_t
{

} mpipe_net_serial_params_t;

typedef struct mpipe_net_transport_params_t
{
    enum mpipe_net_transport_id_t           transport_id;           // идентификатор транспорта
    union
    {
        mpipe_net_tcp_params_t              tcp;
        mpipe_net_udp_params_t              udp;
        mpipe_net_serial_params_t           serial;
    }                                       params;                 // блок параметров зависящий от транспорта
    void*                                   user_data;              // пользовательский контекст
    mpipe_net_transport_cb_t                cb;
} mpipe_net_transport_params_t;

// создать транспорт и получить его дескриптор
EXPORT_API mpipe_handle_t mpipe_net_transport_create(const struct mpipe_net_transport_params_t* params);

// задать карту обратных вызовов для транспорта
EXPORT_API enum mpipe_result_t mpipe_net_transport_set_cb(mpipe_handle_t handle
                                                            , const struct mpipe_net_transport_cb_t* transport_cb);

// управление транспортом (открыть/запустить/остановить/закрыть)
EXPORT_API enum mpipe_result_t mpipe_net_transport_control(mpipe_handle_t handle
                                                            , enum mpipe_object_control_id_t control_id);

// получить информацию о транспорте (считать описатель с текущим состоянием)
EXPORT_API enum mpipe_result_t mpipe_net_transport_get_info(mpipe_handle_t handle
                                                            , struct mpipe_net_transport_t* transport);

// отправить пакет
EXPORT_API enum mpipe_result_t mpipe_net_transport_send_packet(mpipe_handle_t handle
                                                                , const struct mpipe_net_packet_t* packet);



EXTERN_C_END

#endif // MPIPE_API_NET_H
